﻿using C1.DataCollection;
using C1.DataCollection.Serialization;
using System.Net.Http.Json;
using System.Text.Json;
using FlexGridIntro.Shared;
using System.Threading;

namespace FlexGridIntro.Client.Models
{
    public class VirtualDataCollection<T> : C1VirtualDataCollection<T> where T : class
    {
        private readonly HttpClient Http;
        private readonly string URL;
        private int TotalCount;
        public VirtualDataCollection(HttpClient http, string apiURL) 
        {
            this.Http = http;
            this.URL = apiURL;
            
            
        }

        public override bool CanInsert(int index, T item)
        {
            return true;
            //return base.CanInsert(index, item);
        }
        protected override async Task<int> InsertAsyncOverride(int index, T item, CancellationToken cancellationToken)
        {
            var res = await Http.PostAsJsonAsync<T>(new Uri($"{URL}", UriKind.Relative), item);
            return index;
        }
        public override bool CanSort(params SortDescription[] sortDescriptions)
        {
            return true;
        }

        public override bool CanRemove(int index)
        {
            return true;   
        }
        public override bool CanFilter(FilterExpression? filterExpression)
        {
            return !(filterExpression is FilterPredicateExpression);
        }

        
        protected override async Task<Tuple<int, IReadOnlyList<T>>> GetPageAsync(int pageIndex, int startingIndex, int count, IReadOnlyList<SortDescription>? sortDescriptions = null, FilterExpression? filterExpression = null, CancellationToken cancellationToken = default)
        {
            string url = $"{URL}?skip={startingIndex}&take={count}";
            
            if(this.TotalCount == 0)
            {
                // Get TotalCount to virtualize Data; This method should be define in the API
                this.TotalCount = await Http.GetFromJsonAsync<int>(new Uri($"{URL}/totalcount", UriKind.Relative), cancellationToken);
            }
            if (sortDescriptions?.Count > 0)
            {
                url += $"&sort={Uri.EscapeDataString(JsonSerializer.Serialize<IReadOnlyList<SortDescription>>(sortDescriptions))}";
            }
            if (filterExpression != null)
            {
                var options = new JsonSerializerOptions { Converters = { new FilterExpressionJsonConverter() } };
                url += $"&filter={Uri.EscapeDataString(JsonSerializer.Serialize(filterExpression, options))}";
            }
            var response = await Http.GetFromJsonAsync<IEnumerable<T>>(new Uri(url, UriKind.Relative), cancellationToken);
            Console.WriteLine($"From API Class {startingIndex} to {startingIndex+count}");
            return new Tuple<int, IReadOnlyList<T>>(TotalCount,response.ToList());

        }
    }
}
