﻿using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using FlexGridIntro.Shared;
using C1.DataCollection;
using C1.DataCollection.Serialization;
using System.Text.Json;

namespace FlexGridIntro.Server.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class CustomerController : ControllerBase
    {
        private static IEnumerable<Customers> _customers = Customers.GetCustomers(200);
        private readonly ILogger<CustomerController> logger;
        public CustomerController(ILogger<CustomerController> logger)
        {
            this.logger = logger;
        }

        [HttpGet("totalcount")]
        public async Task<int> GetCount()
        {
            return _customers.Count();
        }

        [HttpGet]
        public async Task<IEnumerable<Customers>> Get()
        {
            var skip = 0;
            var take = 10;
            int.TryParse(Request.Query?["skip"].FirstOrDefault(), out skip);
            int.TryParse(Request.Query?["take"].FirstOrDefault(), out take);

            var customers = _customers.ToList();

            #region filter
            var filter = Request.Query?["filter"].FirstOrDefault();

            if (!string.IsNullOrWhiteSpace(filter))
            {
                var options = new JsonSerializerOptions { Converters = { new FilterExpressionJsonConverter() } };
                var filterExpression = JsonSerializer.Deserialize<FilterExpression>(filter, options);
                var filterCollection = new C1FilterDataCollection<Customers>(customers);
                await filterCollection.FilterAsync(filterExpression);
                customers = filterCollection.ToList();
            }
            #endregion

            #region sorting
            var sort = Request.Query?["sort"].FirstOrDefault();

            if (!string.IsNullOrWhiteSpace(sort))
            {
                var options = new JsonSerializerOptions { Converters = { new SortDescriptionJsonConverter() } };
                var sortDescriptions = JsonSerializer.Deserialize<SortDescription[]>(sort, options);
                var sortCollection = new C1SortDataCollection<Customers>(customers);
                await sortCollection.SortAsync(sortDescriptions);
                customers = sortCollection.ToList();
            }
            #endregion
            return customers.Skip(skip).Take(take);//new CustomerResponse { TotalCount = customers.Count, Customers = customers.Skip(skip).Take(take) };
        }

        [HttpPost]
        public async Task Get(Customers customer)
        {
            try
            {
                _customers.Append(customer);
                
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }
    }
}
