﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Reflection;
using System.Runtime.CompilerServices;
using System.Text;
using System.Text.Json.Serialization;
using System.Threading.Tasks;

namespace FlexGridIntro.Shared
{
    public class Customers : INotifyPropertyChanged, IEditableObject
    {
        #region Fields

        int _id, _countryId, _orderCount;
        string _first, _last;
        string _address, _city, _postalCode, _email;
        bool _active;
        DateTime _lastOrderDate;
        double _orderTotal;

        #endregion

        #region ** object model

        public int OrderCount
        {
            get { return _orderCount; }
            set
            {
                if (value != _orderCount)
                {
                    _orderCount = value;
                    OnPropertyChanged();
                }
            }
        }

        [Display(AutoGenerateField = false)]
        [JsonIgnore]
        public double OrderTotal
        {
            get { return _orderTotal; }
            set
            {
                if (value != _orderTotal)
                {
                    _orderTotal = value;
                    OnPropertyChanged();
                }
            }
        }

        [Display(AutoGenerateField = false)]
        [JsonIgnore]
        public bool Active
        {
            get { return _active; }
            set
            {
                if (value != _active)
                {
                    _active = value;
                    OnPropertyChanged();
                }
            }
        }

        [Display(AutoGenerateField = false)]
        [JsonIgnore]
        public string Name
        {
            get { return string.Format("{0} {1}", FirstName, LastName); }
        }

        
        [Display(AutoGenerateField = false)]
        [JsonIgnore]
        public double OrderAverage
        {
            get { return OrderTotal / (double)OrderCount; }
        }
        [Display(AutoGenerateField = false)]
        //[JsonIgnore]
        public int Id
        {
            get { return _id; }
            set
            {
                if (value != _id)
                {
                    _id = value;
                    OnPropertyChanged();
                }
            }
        }

        public string FirstName
        {
            get { return _first; }
            set
            {
                if (value != _first)
                {
                    _first = value;
                    OnPropertyChanged();
                    OnPropertyChanged("Name");
                }
            }
        }

        public string LastName
        {
            get { return _last; }
            set
            {
                if (value != _last)
                {
                    _last = value;
                    OnPropertyChanged();
                    OnPropertyChanged("Name");
                }
            }
        }

        public string Address
        {
            get { return _address; }
            set
            {
                if (value != _address)
                {
                    _address = value;
                    OnPropertyChanged();
                }
            }
        }

        private string _link;
        public string Link
        {
            get { return _link ; }
            set
            {
                if (value != _link)
                {
                    _link = value;
                    OnPropertyChanged();
                }
            }
        }

        public string City
        {
            get { return _city; }
            set
            {
                if (value != _city)
                {
                    _city = value;
                    OnPropertyChanged();
                }
            }
        }

       
        public string PostalCode
        {
            get { return _postalCode; }
            set
            {
                if (value != _postalCode)
                {
                    _postalCode = value;
                    OnPropertyChanged();
                }
            }
        }

        [Display(Name = "e-mail")]
        public string Email
        {
            get { return _email; }
            set
            {
                if (value != _email)
                {
                    _email = value;
                    OnPropertyChanged();
                }
            }
        }

        [Display(AutoGenerateField = false)]
        [JsonIgnore]
        public DateTime LastOrderDate
        {
            get { return _lastOrderDate; }
            set
            {
                if (value != _lastOrderDate)
                {
                    _lastOrderDate = value;
                    OnPropertyChanged();
                }
            }
        }
        #endregion
        #region Data Method
        public static ObservableCollection<Customers> GetCustomers(int Count)
        {
            var _names = "Alex,Bob,Chris,David,Evan,Freddy,George,Henry,Isac,Jack,Ken,Laurie".Split(",");
            var _buyingGroups = ",WJ,SP,C1,AR,".Split(",");
            var _categories = "Platinum,Standard".Split(",");
            var _random = new Random();
            var data = new ObservableCollection<Customers>();
            for (int i = 0; i < Count; i++)
            {
                var _rnd = _random.Next();
                data.Add(new Customers()
                {
                    Active = i%2 == 0,
                    FirstName = _names[i % _names.Length],
                    LastName = _names[i % _names.Length],
                    LastOrderDate = DateTime.Now.AddDays(i),
                    Id=i,
                    Link = "counter"
                    
                });
                
            }
            //Thread.Sleep(2000); // simulate network delay 
            return data;


        }
        #endregion

        #region ** INotifyPropertyChanged Members

        // this interface allows bounds controls to react to changes in the data objects.
        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged([CallerMemberName] string propertyName = "")
        {
            OnPropertyChanged(new PropertyChangedEventArgs(propertyName));
        }

        protected void OnPropertyChanged(PropertyChangedEventArgs e)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, e);
        }

        #endregion

        #region IEditableObject Members

        // this interface allows transacted edits (user can press escape to restore previous values).

        Customers _clone;
        public void BeginEdit()
        {
            _clone = (Customers)this.MemberwiseClone();
        }

        public void EndEdit()
        {
            _clone = null;
        }

        public void CancelEdit()
        {
            if (_clone != null)
            {
                foreach (var p in this.GetType().GetRuntimeProperties())
                {
                    if (p.CanRead && p.CanWrite)
                    {
                        p.SetValue(this, p.GetValue(_clone, null), null);
                    }
                }
            }
        }

        #endregion

    }
    public class TResponse<T> where T:class
    {
        public int TotalCount { get; set; }
        public IEnumerable<T> Responses { get; set; }
    }
    public class CustomerResponse
    {
        public int TotalCount { get; set; }
        public IEnumerable<Customers> Customers { get; set; }
    }
}
